#include "stdafx.h"
#include "NeMessenger.h"
#include "InfoBannerWnd.h"
#include "XHyperLink.h"
#include "httpclient.h"
#include "main_frame.hpp"


CInfoBannerController::CInfoBannerController()
: m_nRollingCount(0)
, m_pCachedBitmap(NULL)
, m_sizeBannerImage(0, 0)
, m_nCurBannerSeq(0)
,m_pFrame(NULL)
{
	m_hLinkCursor = AfxGetApp()->LoadStandardCursor(IDC_HAND);
}

CInfoBannerController::~CInfoBannerController()
{
	if( m_pCachedBitmap != NULL )
		delete m_pCachedBitmap;

	if (m_hLinkCursor)
	{
		DestroyCursor(m_hLinkCursor);
		m_hLinkCursor = NULL;
	}
}


bool CInfoBannerController::DrawBanner(unsigned long nRollingCount)
{
	if( m_mapBannerList.empty() )
		return false;

	NE_BANNER_DATA banner = GetBannerData(nRollingCount);
	if( IsAvailableBanner(banner) != true )
		return false;

	if( m_nCurBannerSeq == banner.nSeq )
		return false;

	// banner image file
	CString strBannerFilePath = GetBannerFilePath(nRollingCount);
	if( strBannerFilePath.IsEmpty() )
		return false;
	m_nCurBannerSeq = banner.nSeq;
    m_pFrame->SetBannerControl(strBannerFilePath);

	return true;
}


void CInfoBannerController::SetBannerList(const std::vector<NE_BANNER_DATA> &vecBannerList)
{
	if( vecBannerList.empty() )
		return;

	m_pFrame->KillTimer(TID_BANNER_ROLLING);

	std::vector<NE_BANNER_DATA> vecAddedBanner;

	m_mapBannerList.clear();
	std::vector<NE_BANNER_DATA>::const_iterator itr = vecBannerList.begin();
	for(; itr != vecBannerList.end(); itr++)
	{
		m_mapBannerList.insert(make_pair(itr->nSeq, *itr));
		vecAddedBanner.push_back(*itr);
	}

	std::vector<NE_BANNER_DATA>::const_iterator iter = vecAddedBanner.begin();
	for(; iter != vecAddedBanner.end(); iter++)
	{
		
		WSTRING strServerAddr = g_pMessengerActor->GetWASActor()->GetWASServerAddress();
		//strBannerImgUrl.Format(_T("http://biz-ntalk.ncloudbiz.com%s"), iter->strBannerImageURL.c_str());
		WSTRING strBannerImgUrl = strServerAddr + iter->strBannerImageURL;
		NE_BANNER_DATA *pBannerData = new NE_BANNER_DATA;
		if( pBannerData == NULL )
			continue;

		pBannerData->nSeq = iter->nSeq;
		pBannerData->strTitle = iter->strTitle;
		pBannerData->strBannerImageURL = strBannerImgUrl;
		pBannerData->strBannerContentURL = iter->strBannerContentURL;
		pBannerData->strStartDate = iter->strStartDate;
		pBannerData->strEndDate = iter->strEndDate;

		CWinThread *pThread = AfxBeginThread(ThreadDownloadBannerImage, pBannerData, THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
		if (pThread == NULL)
		{
			TRACE(_T("Failed to create banner image download thread !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"));	
			delete pBannerData;
		}
		else
		{
			pThread->ResumeThread();
		}
	}
	if( m_nRollingCount == 0)
		m_pFrame->SetTimer(TID_BANNER_ROLLING, 2000);
	else
		m_pFrame->SetTimer(TID_BANNER_ROLLING, BANNER_ROLLING_INTERVAL);
}

UINT CInfoBannerController::ThreadDownloadBannerImage(LPVOID pParam)
{
	if( pParam == NULL )
		return 0;

	NE_BANNER_DATA *pBannerData = (NE_BANNER_DATA*)pParam;

	CString strBannerDirPath;
	strBannerDirPath.Format(_T("%s\\Banner\\"), GET_APP()->GetAppDataDir());

	if(_waccess(strBannerDirPath, 0) == -1)
		_wmkdir(strBannerDirPath);

	std::wstring strFileName = pBannerData->GetBannerFileName();

	CString strBannerImgFileName;
	strBannerImgFileName.Format(_T("%s"), strFileName.c_str());
	strBannerDirPath.Append(strBannerImgFileName);
	if( PathFileExists(strBannerDirPath) )
	{
		delete pBannerData;
		return 0;
	}

	httpbase::GenericHTTPClient httpClient;

	httpClient.Request(NULL, pBannerData->strBannerImageURL.c_str(), httpbase::GenericHTTPClient::RequestGetMethod, strBannerDirPath);

	delete pBannerData;

	return 0;
}



void CInfoBannerController::OnTimer(UINT_PTR nIDEvent)
{
	if( nIDEvent == TID_BANNER_ROLLING )
	{
		m_pFrame->KillTimer(TID_BANNER_ROLLING);

		m_nRollingCount++;
		DrawBanner(m_nRollingCount);
        m_pFrame->SetTimer(TID_BANNER_ROLLING, BANNER_ROLLING_INTERVAL);
	}

}

NE_BANNER_DATA CInfoBannerController::GetBannerData(unsigned long nRollingCount)
{
	if( m_mapBannerList.empty() )
		return NE_BANNER_DATA();

	unsigned int nIndex = nRollingCount % m_mapBannerList.size();
	std::map<int, NE_BANNER_DATA>::iterator itr = m_mapBannerList.begin();
	for( unsigned int i = 0; i < nIndex; i++)
		itr++;

	return itr->second;
}

CString CInfoBannerController::GetBannerFilePath(unsigned long nRollingCount)
{
	NE_BANNER_DATA banner = GetBannerData(nRollingCount);
	if( banner.strBannerImageURL.empty() )
		return CString();

	CString strBannerImgPath;
	strBannerImgPath.Format(_T("%s\\Banner\\%s"), GET_APP()->GetAppDataDir(), banner.GetBannerFileName().c_str());
	if( ::PathFileExists(strBannerImgPath) != TRUE )
		return CString();

	return strBannerImgPath;
}

bool CInfoBannerController::IsAvailableBanner(const NE_BANNER_DATA &banner)
{
	CString strStartDate(banner.strStartDate.c_str());
	strStartDate.Replace(_T("."), _T(""));
	CString strEndDate(banner.strEndDate.c_str());
	strEndDate.Replace(_T("."), _T(""));

	CTime time(CTime::GetCurrentTime());
	GetLocaleTime(&time);

	CString strDateTime;
	strDateTime.Format(_T("%04d%02d%02d"), time.GetYear(), time.GetMonth(), time.GetDay());

	if( _wtoi(strStartDate) > _wtoi(strDateTime) )
		return false;

	if( _wtoi(strEndDate) < _wtoi(strDateTime) )
		return false;

	return true;
}

// URL ̵ LButtonUp() óϸ ȵȴ.
// ֳϸ, Ntalk â  Ʒ      ˾(/ä ȳ ) ߴ 찡 ִµ
// ˾ LButtonDown ˾ , LButtonUp ʰ ŬǾ ũ   ߻Ѵ.
void CInfoBannerController::OnLButtonDown()
{
	if( m_nCurBannerSeq > 0 )
	{
		std::map<int, NE_BANNER_DATA>::iterator itr = m_mapBannerList.find(m_nCurBannerSeq);
		if( itr != m_mapBannerList.end() 
			&& false == itr->second.strBannerContentURL.empty())
			CXHyperLink::GotoURL(itr->second.strBannerContentURL.c_str(), SW_SHOWNORMAL);
	}

}
